from enum import StrEnum


class DataColumnName(StrEnum):
    PARAMETER0 = 'parameter0'
    PARAMETER1 = 'parameter1'
    PARAMETER2 = 'parameter2'
    PARAMETER3 = 'parameter3'
    PARAMETER4 = 'parameter4'
    PARAMETER5 = 'parameter5'
    PARAMETER6 = 'parameter6'
    PARAMETER7 = 'parameter7'
    PARAMETER8 = 'parameter8'
    PARAMETER9 = 'parameter9'
    PARAMETER10 = 'parameter10'
    PHASE_AMPLITUDE0 = 'phase_amplitude0'
    PHASE_AMPLITUDE1 = 'phase_amplitude1'
    PHASE_AMPLITUDE2 = 'phase_amplitude2'
    PHASE_AMPLITUDE3 = 'phase_amplitude3'
    PHASE_AMPLITUDE4 = 'phase_amplitude4'
    PHASE_AMPLITUDE5 = 'phase_amplitude5'
    PHASE_AMPLITUDE6 = 'phase_amplitude6'
    PHASE_AMPLITUDE7 = 'phase_amplitude7'
    PHASE_AMPLITUDE8 = 'phase_amplitude8'
    PHASE_AMPLITUDE9 = 'phase_amplitude9'
    PHASE_AMPLITUDE10 = 'phase_amplitude10'
    PHASE_AMPLITUDE11 = 'phase_amplitude11'
    PHASE_AMPLITUDE12 = 'phase_amplitude12'
    PHASE_AMPLITUDE13 = 'phase_amplitude13'
    PHASE_AMPLITUDE14 = 'phase_amplitude14'
    PHASE_AMPLITUDE15 = 'phase_amplitude15'
    PHASE_AMPLITUDE16 = 'phase_amplitude16'
    PHASE_AMPLITUDE17 = 'phase_amplitude17'
    PHASE_AMPLITUDE18 = 'phase_amplitude18'
    PHASE_AMPLITUDE19 = 'phase_amplitude19'
    PHASE_AMPLITUDE20 = 'phase_amplitude20'
    PHASE_AMPLITUDE21 = 'phase_amplitude21'
    PHASE_AMPLITUDE22 = 'phase_amplitude22'
    PHASE_AMPLITUDE23 = 'phase_amplitude23'
    PHASE_AMPLITUDE24 = 'phase_amplitude24'
    PHASE_AMPLITUDE25 = 'phase_amplitude25'
    PHASE_AMPLITUDE26 = 'phase_amplitude26'
    PHASE_AMPLITUDE27 = 'phase_amplitude27'
    PHASE_AMPLITUDE28 = 'phase_amplitude28'
    PHASE_AMPLITUDE29 = 'phase_amplitude29'
    PHASE_AMPLITUDE30 = 'phase_amplitude30'
    PHASE_AMPLITUDE31 = 'phase_amplitude31'
    PHASE_AMPLITUDE32 = 'phase_amplitude32'
    PHASE_AMPLITUDE33 = 'phase_amplitude33'
    PHASE_AMPLITUDE34 = 'phase_amplitude34'
    PHASE_AMPLITUDE35 = 'phase_amplitude35'
    PHASE_AMPLITUDE36 = 'phase_amplitude36'
    PHASE_AMPLITUDE37 = 'phase_amplitude37'
    PHASE_AMPLITUDE38 = 'phase_amplitude38'
    PHASE_AMPLITUDE39 = 'phase_amplitude39'
    PHASE_AMPLITUDE40 = 'phase_amplitude40'
    PHASE_AMPLITUDE41 = 'phase_amplitude41'
    PHASE_AMPLITUDE42 = 'phase_amplitude42'
    PHASE_AMPLITUDE43 = 'phase_amplitude43'
    PHASE_AMPLITUDE44 = 'phase_amplitude44'
    PHASE_AMPLITUDE45 = 'phase_amplitude45'
    PHASE_AMPLITUDE46 = 'phase_amplitude46'
    PHASE_AMPLITUDE47 = 'phase_amplitude47'
    PHASE_AMPLITUDE48 = 'phase_amplitude48'
    PHASE_AMPLITUDE49 = 'phase_amplitude49'
    PHASE_AMPLITUDE50 = 'phase_amplitude50'
    PHASE_AMPLITUDE51 = 'phase_amplitude51'
    PHASE_AMPLITUDE52 = 'phase_amplitude52'
    PHASE_AMPLITUDE53 = 'phase_amplitude53'
    PHASE_AMPLITUDE54 = 'phase_amplitude54'
    PHASE_AMPLITUDE55 = 'phase_amplitude55'
    PHASE_AMPLITUDE56 = 'phase_amplitude56'
    PHASE_AMPLITUDE57 = 'phase_amplitude57'
    PHASE_AMPLITUDE58 = 'phase_amplitude58'
    PHASE_AMPLITUDE59 = 'phase_amplitude59'
    PHASE_AMPLITUDE60 = 'phase_amplitude60'
    PHASE_AMPLITUDE61 = 'phase_amplitude61'
    PHASE_AMPLITUDE62 = 'phase_amplitude62'
    PHASE_AMPLITUDE63 = 'phase_amplitude63'

